using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using FlickrNet;

namespace LiveWriterPlugin
{
    public partial class FlickrBrowseForm : Form
    {
        public FlickrBrowseForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Result as supplied to the user of the form. 
        /// </summary>
        public string ResultString = string.Empty;

        /// <summary>
        /// Link to the Flickr session. Preserved over several 
        /// uses of the form to reduce network traffic.
        /// </summary>
        private Flickr flickr = null;

        /// <summary>
        /// Username for the flickr account. Entered into the 
        /// form and copied here. Used to make sure that we
        /// only update the pictures when the username changes.
        /// </summary>
        private string userName = string.Empty;

        /// <summary>
        /// My api key. I got it from Flickr. If you want to write your
        /// own programs I'd appreciate it if you got your own.
        /// </summary>
        private string apiKey = "840b4ae02668f432077020f1f4f40087";

        /// <summary>
        /// The user that we are using for this session.
        /// </summary>
        private FoundUser user;

        /// <summary>
        /// Options for search. At present I get the pictures in
        /// the order of most recently uploaded.
        /// </summary>
        private PhotoSearchOptions userSearch;

        /// <summary>
        /// The photos that I get back. For reasons of laziness this
        /// is only the most recent 100 that I've posted
        /// </summary>
        private Photos usersPhotos;

        /// <summary>
        /// Currently selected photo property value.
        /// </summary>
        private Photo thisPhotoValue;

        /// <summary>
        /// Currently selected property. Setting this also
        /// causes the browser view to update.
        /// </summary>
        private Photo thisPhoto
        {
            get
            {
                return thisPhotoValue;
            }
            set
            {
                thisPhotoValue = value;
                previewWebBrowser.Url =
                    new Uri(thisPhotoValue.SmallUrl);
            }
        }

        /// <summary>
        /// Picture number property value.
        /// </summary>
        private int pictureNoVal;

        /// <summary>
        /// Current position in the 100 shots we have 
        /// downloaded. Changing this value sets the 
        /// presently selected photo and also updates
        /// the state of the movement buttons.
        /// </summary>
        private int pictureNo
        {
            get
            {
                return pictureNoVal;
            }
            set
            {
                /// Validate
                if (value < 0)
                {
                    return;
                }

                if (value >= usersPhotos.PhotosPerPage)
                {
                    return;
                }

                pictureNoVal = value;

                thisPhoto = usersPhotos.PhotoCollection[pictureNoVal];

                if (pictureNoVal == 0)
                {
                    goLeftButton.Enabled = false;
                }
                else
                {
                    goLeftButton.Enabled = true;
                }

                if (pictureNoVal >= (usersPhotos.PhotosPerPage-1))
                {
                    goRightButton.Enabled = false;
                }
                else
                {
                    goRightButton.Enabled = true;
                }
            }
        }

        /// <summary>
        /// Turns off all the buttons. Used when we haven't got
        /// any pictures to move through/select.
        /// </summary>
        private void disableImageButtons()
        {
            goLeftButton.Enabled = false;
            goRightButton.Enabled = false;
            doneButton.Enabled = false;
        }

        /// <summary>
        /// Loads the images for a given Flickr user. The first 
        /// time it is called it will also create a flickr session.
        /// </summary>
        private void doLoadImages()
        {
            // force intial load 
            if (flickr == null)
            {
                try
                {
                    flickr = new FlickrNet.Flickr(apiKey);
                }
                catch (FlickrException flickerException)
                {
                    MessageBox.Show(flickerException.Message);
                    disableImageButtons();
                    return;
                }
            }

            /// if we have a new username we need to login and
            /// load some pictures.
            
            if (usernameTextBox.Text != userName)
            {
                try
                {
                    user = flickr.PeopleFindByUsername(usernameTextBox.Text);
                    userName = usernameTextBox.Text;
                }
                catch (FlickrException flickerException)
                {
                    MessageBox.Show(flickerException.Message);
                    return;
                }
            }

            /// Build the search option
            userSearch = new PhotoSearchOptions();
            userSearch.UserId = user.UserId;
            userSearch.SortOrder = FlickrNet.PhotoSearchSortOrder.DateTakenDesc;

            /// Go get the pictures
            usersPhotos = flickr.PhotosSearch(userSearch);

            /// display the most recent photo and set up for navigation
            pictureNo = 90;

            /// allow pictures to be selected
            doneButton.Enabled = true;
        }

        /// <summary>
        /// This the width to which our pictures will be scaled. It would be 
        /// trivial to add to our form to allow this to be changed.
        /// </summary>
        private int desiredPhotoWidth = 400;

        /// <summary>
        /// Completes the load of the picture. Creates the html and
        /// closes the form down.
        /// </summary>
        /// <param name="width">width in pixels of the desired image.</param>
        private void doDone( int width )
        {
            /// Get the sizes of the image
            Sizes sizes = flickr.PhotosGetSizes(thisPhoto.PhotoId);

            /// Pull out what should be the biggest size
            FlickrNet.Size s = sizes.SizeCollection[sizes.SizeCollection.Length - 1];

            /// Calculate the aspect ratio, for scaling.
            float aspect = (float)s.Width / (float)s.Height;

            /// Work out the image height
            int height = (int)((float)width / aspect);

            /// Build some HTML using the medium sized image as a starting point
            /// Insert the image as a link (which is how Flickr prefer it)
            this.ResultString = @"<a href=""" +
                thisPhoto.WebUrl +
                @""" title=""Photo Sharing""><img src=""" +
                thisPhoto.MediumUrl +
                @""" width=""" + width +
                @""" height=""" + height +
                @""" alt=""" +
                thisPhoto.PhotoId +
                @"""  /></a>";

            /// Report happy ending
            this.DialogResult = DialogResult.OK;

            Close();
        }

        private void doneButton_Click(object sender, EventArgs e)
        {
            doDone(desiredPhotoWidth);
        }

        private void loadButton_Click(object sender, EventArgs e)
        {
            doLoadImages();
        }

        private void goLeftButton_Click(object sender, EventArgs e)
        {
            pictureNo = pictureNo - 1;
        }

        private void goRightButton_Click(object sender, EventArgs e)
        {
            pictureNo = pictureNo + 1;
        }
    }
}